@@ -6,6 +6,23 @@ import numpy as modded_np
 cls = VecValDer
 
 
+# On 5-Stage Ring Oscillator example from PyMAPP, using this instead of
+# np.ndindex reduced a 200 s simulation to 150 s.
+def _ndindex(shape):
+    state = [0 for _ in shape]
+    total = 0
+    stop_condition = np.product(shape)
+    while total < stop_condition:
+        total += 1
+        yield tuple(state)
+        for i in range(-1, -1 -len(shape), -1):
+            state[i] += 1
+            if state[i] == shape[i]:
+                state[i] = 0
+            else:
+                break
+
+
 class FeaturelessVecValDer:
     __slots__ = 'val', 'der'
     def __init__(self, val, der):
@@ -30,7 +47,6 @@ def _derivative_independent_comparison_ufunc(fn):
 
 
 _none_vec_val_der = FeaturelessVecValDer(None, None)
-# Define a decorator which adds out support, erroring on non-vecvalder  
 
 
 # This is probably a good C/Cython candidate
@@ -259,9 +275,9 @@ def positive(x, /, out):
 @_add_out_support
 def add(x1, x2, /, out):
     broadcast_obj = np.broadcast(x1, x2)
-    idx_iterator = modded_np.ndindex(broadcast_obj.shape)
     if out.val is None:
         out = modded_np.empty(broadcast_obj.shape)
+    idx_iterator = _ndindex(out.val.shape)
  
     if isinstance(x1, cls) and isinstance(x2, cls):
         for idx, (x1_elem, x2_elem) in zip(idx_iterator, broadcast_obj):
@@ -288,9 +304,9 @@ def add(x1, x2, /, out):
 @_add_out_support
 def subtract(x1, x2, /, out):
     broadcast_obj = np.broadcast(x1, x2)
-    idx_iterator = modded_np.ndindex(broadcast_obj.shape)
     if out.val is None:
         out = modded_np.empty(broadcast_obj.shape)
+    idx_iterator = _ndindex(out.val.shape)
  
     if isinstance(x1, cls) and isinstance(x2, cls):
         for idx, (x1_elem, x2_elem) in zip(idx_iterator, broadcast_obj):
@@ -318,9 +334,9 @@ def subtract(x1, x2, /, out):
 @_add_out_support
 def multiply(x1, x2, /, out):
     broadcast_obj = np.broadcast(x1, x2)
-    idx_iterator = modded_np.ndindex(broadcast_obj.shape)
     if out.val is None:
         out = modded_np.empty(broadcast_obj.shape)
+    idx_iterator = _ndindex(out.val.shape)
  
     if isinstance(x1, cls) and isinstance(x2, cls):
         for idx, (x1_elem, x2_elem) in zip(idx_iterator, broadcast_obj):
@@ -349,9 +365,9 @@ def multiply(x1, x2, /, out):
 @_add_out_support
 def true_divide(x1, x2, /, out):
     broadcast_obj = np.broadcast(x1, x2)
-    idx_iterator = modded_np.ndindex(broadcast_obj.shape)
     if out.val is None:
         out = modded_np.empty(broadcast_obj.shape)
+    idx_iterator = _ndindex(out.val.shape)
  
     if isinstance(x1, cls) and isinstance(x2, cls):
         for idx, (x1_elem, x2_elem) in zip(idx_iterator, broadcast_obj):
@@ -380,9 +396,9 @@ def true_divide(x1, x2, /, out):
 @_add_out_support
 def float_power(x1, x2, /, out):
     broadcast_obj = np.broadcast(x1, x2)
-    idx_iterator = modded_np.ndindex(broadcast_obj.shape)
     if out.val is None:
         out = modded_np.empty(broadcast_obj.shape)
+    idx_iterator = _ndindex(out.val.shape)
  
     if isinstance(x1, cls) and isinstance(x2, cls):
         for idx, (x1_elem, x2_elem) in zip(idx_iterator, broadcast_obj):
@@ -410,9 +426,9 @@ def float_power(x1, x2, /, out):
 @_add_out_support
 def power(x1, x2, /, out):
     broadcast_obj = np.broadcast(x1, x2)
-    idx_iterator = modded_np.ndindex(broadcast_obj.shape)
     if out.val is None:
         out = modded_np.empty(broadcast_obj.shape)
+    idx_iterator = _ndindex(out.val.shape)
  
     if isinstance(x1, cls) and isinstance(x2, cls):
         for idx, (x1_elem, x2_elem) in zip(idx_iterator, broadcast_obj):
@@ -448,7 +464,7 @@ def matmul(x1, x2, /, out):
             der = np.ndarray((*val.shape, *x1.der.shape[-2:]))
         else:
             der = out.der
-        for i, k in modded_np.ndindex(val.shape):
+        for i, k in _ndindex(val.shape):
             der[i, k] = sum(
                 x1.val[i, j] * x2.der[j, k] + x1.der[i, j] * x2.val[j, k]
                 for j in range(x1.shape[1]))
@@ -462,7 +478,7 @@ def matmul(x1, x2, /, out):
             der = np.ndarray((*val.shape, *x1.der.shape[-2:]))
         else:
             der = out.der
-        for i, k in modded_np.ndindex(val.shape):
+        for i, k in _ndindex(val.shape):
             der[i, k] = sum(x1.der[i, j] * x2[j, k]
                             for j in range(x1.shape[1]))
         return cls(val, der)
@@ -475,7 +491,7 @@ def matmul(x1, x2, /, out):
             der = np.ndarray((*val.shape, *x2.der.shape[-2:]))
         else:
             der = out.der
-        for i, k in modded_np.ndindex(val.shape):
+        for i, k in _ndindex(val.shape):
             der[i, k] = sum(x1[i, j] * x2.der[j, k]
                             for j in range(x1.shape[1]))
         return cls(val, der)
